<?php

namespace LakeDrops\DrupalEnvironment;

use Exception;
use LakeDrops\Component\Composer\BaseHandler;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Class Handler.
 *
 * @package LakeDrops\DrupalEnvironment
 */
class Handler extends BaseHandler {

  /**
   * {@inheritdoc}
   */
  public function configId(): string {
    return 'lakedrops';
  }

  /**
   * {@inheritdoc}
   */
  protected function configDefault(): array {
    return [
        'chg-acl' => FALSE,
        'webserver-username' => 'www-data',
        'webserver-groupname' => 'www-data',
        'filter_protocols' => [
          'http',
          'https',
          'ftp',
          'news',
          'nntp',
          'tel',
          'telnet',
          'mailto',
          'irc',
          'ssh',
          'sftp',
          'webcal',
          'rtsp',
        ],
        'cors_config' => [
          'enabled' => 'false',
          'allowedOrigins' => '*',
        ],
        'db' => [
          'driver' => 'mysql',
          'namespace' => 'Drupal\Core\Database\Driver\mysql',
          'database' => 'example',
          'username' => 'username',
          'password' => 'password',
          'host' => '127.0.0.1',
          'port' => '3306',
          'prefix' => '',
        ],
        'domain' => str_replace(['.', '-'], ['\\.', '\\-'], 'www.example.com'),
        'aliases' => [],
        'live' => !empty($this->env->receiveGlobal('DRUPAL_LIVE', 'Live mode', '0')),
      ];
  }

  /**
   * Post project create event to setup Drupal environment.
   */
  public function setupLakeDropsProject() {
    $this->init();

    $fs = new Filesystem();
    $installationManager = $this->composer->getInstallationManager();

    $drupalCorePackage = $this->getDrupalCorePackage();
    $corePath = $installationManager->getInstallPath($drupalCorePackage);
    $this->config->setValue('drupal8', version_compare($drupalCorePackage->getVersion(), '9.0.0', '<='), FALSE);

    // Directory where the root project is being created.
    $projectRoot = getcwd();
    // Directory where Drupal's index.php is located.
    $webRoot = dirname($corePath);
    // Directory where this plugin is being installed.
    $pluginRoot = $installationManager->getInstallPath($this->getPackage('lakedrops/drupal-environment'));

    // Create and protect config, settings and files directories.
    foreach ([
      'settings/default',
      'files/default/files',
      'files/default/files/css',
      'files/default/files/js',
      'files/default/files/languages',
      'files/default/files/translations',
      'files/default/private',
      'drush',
    ] as $dir) {
      $path = $projectRoot . '/' . $dir;
      if (!$fs->exists($path)) {
        $fs->mkdir($path);
        $fs->copy($pluginRoot . '/templates/.htaccess', $path . '/.htaccess');
      }
      if ($this->config->readValue('chg-acl')) {
        $fs->chgrp($path, $this->config->readValue('webserver-groupname'), TRUE);
      }
    }

    // Link Drupal site's files and private directory.
    foreach (['files', 'private'] as $dir) {
      if (!$fs->exists($webRoot . '/sites/default/' . $dir)) {
        $rel = substr($fs->makePathRelative($projectRoot . '/files/default/' . $dir, $projectRoot . '/' . $webRoot . '/sites/default/' . $dir), 3, -1);
        $fs->symlink($rel, $webRoot . '/sites/default/' . $dir);
        $fs->chmod($webRoot . '/sites/default/' . $dir, 0775);
      }
    }

    // Link Drupal site's settings files.
    try {
      $hash = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode(random_bytes(55)));
    }
    catch (Exception $e) {
      $hash = 'no-random-bytes-available';
    }
    $this->config->setValue('hash', $hash, FALSE);

    $settings_files = [
      'settings.php',
      'services.yml',
    ];
    if (!$this->isCiContext()) {
      $settings_files[] = 'settings.local.php';
      if (!$this->config->readValue('live')) {
        $settings_files[] = 'development.services.yml';
      }
    }
    $overwrite = $this->isLocalDevMode();
    foreach ($settings_files as $template) {
      $file = $projectRoot . '/settings/default/' . $template;
      $link = $webRoot . '/sites/default/' . $template;
      if ($overwrite || !$fs->exists($link)) {

        $rendered = $this->config->render($template, file_get_contents($pluginRoot . '/templates/' . $template . '.twig'));
        file_put_contents($file, $rendered);

        $rel = substr($fs->makePathRelative($file, $projectRoot . '/' . $link), 3, -1);
        $fs->symlink($rel, $link);
      }
      $fs->chmod($file, 0664);
      if ($this->config->readValue('chg-acl')) {
        $fs->chgrp($file, $this->config->readValue('webserver-groupname'));
      }
    }

    // Link Drupal site's config directory.
    if (!$fs->exists($webRoot . '/sites/default/files/config')) {
      $rel = substr($fs->makePathRelative($projectRoot . '/config/default', $projectRoot . '/files/default/files/config'), 3, -1);
      $fs->symlink($rel, $webRoot . '/sites/default/files/config');
    }

    // Ensure default environment directories for config_split.
    foreach ([
               'develop',
               'test',
               'live',
               'nirvana',
               'snapshot',
             ] as $subdir) {
      $path = $projectRoot . '/config/default/environments/' . $subdir;
      if (!$fs->exists($path)) {
        $fs->mkdir($path);
        $fs->copy($pluginRoot . '/templates/.htaccess', $path . '/.htaccess');
      }
    }
  }

}
